% runAlt2Sec.m 
%      usage: runAlt2Sec
%         by: charlie
%       date: 02/17/2018
%    purpose: run alternation easy/hard. 2 sec long trials
 

function [] = runAlt2Sec(varargin)

% check arguments acvv

if ~any(nargin == [0:10])
    help trOriStim 
    return
end

getArgs(varargin,[],'verbose=1');

if ieNotDefined('initStair'), initStair = 0; end
if ieNotDefined('threshStair1'), threshStair1 = 0; end
if ieNotDefined('threshStair2'), threshStair2 = 5; end
if ieNotDefined('stepsize'), stepsize = 1; end
if ieNotDefined('contrast'), contrast = 1; end
if ieNotDefined('easyTilt'), easyTilt = 20; end
if ieNotDefined('iterationCount'), iterationCount = 1; end
if ieNotDefined('easyhard'), easyhard = 'hard'; end
if ieNotDefined('numberOfRuns'), numberOfRuns = 5; end


% init screen and open up the window
myscreen.background = [255 255 255]/2; %midgray in rgb, corresponds to lightness L of 53.39 in CIElab
myscreen.autoCloseScreen = 0;
myscreen.saveData = 1;
myscreen.displayName = 'Meyer_957_CRT';
%myscreen.displayName = 'laptop';
myscreen = initScreen(myscreen);

% init the task
task{1}.waitForBacktick = 1;
task{1}.seglen = [0.2 1.8]; % stimulus, ITI,
task{1}.getResponse = [0 1];
%task{1}.synchToVol = [0 1];

% Stim1 is the LEFT stimulus
%task{1}.randVars.uniform.tiltStim1 = [-1 1];
% Stim2 is the RIGHT stimulus
task{1}.randVars.uniform.tiltStim2 = [-1 1];

task{1}.numTrials = 75;

for phaseNum = 1:length(task)
    [task{phaseNum} myscreen] = initTask(task{phaseNum},myscreen,@startSegmentCallback,@screenUpdateCallback,@responseCallback);
end

% init the stimulus
global stimulus;
myscreen = initStimulus('stimulus',myscreen);
stimulus.contrast = contrast;
stimulus.easyTilt = easyTilt;

stimulus.easyhard = easyhard;
stimulus.trialCounter = 0;
stimulus.trialCorrectness = [];
stimulus.trialResponse = [];
stimulus.trialRT = [];

stimulus = myInitStimulus(stimulus,myscreen,task);

% init a 2 down 1 up staircase
if initStair == 1 | ieNotDefined('stimulus.stair{1}.threshold') | ieNotDefined('stimulus.stair{2}.threshold')
    disp(sprintf('\nATTENTION: Initalizing new staircase: threshold1 = %0.2f, threshold2 = %0.2f\n', threshStair1, threshStair2));
    stimulus.stair{1} = upDownStaircase(1,2,threshStair1,stepsize,1);
    stimulus.stair{1}.minThreshold = 0;
    stimulus.stair{2} = upDownStaircase(1,2,threshStair2,stepsize,1);
    stimulus.stair{2}.minThreshold = 0;   
else
    disp(sprintf('\nATTENTION: Continuing old staircase: threshold1 = %0.2f, threshold2 = %0.2f\n', stimulus.stair{1}.threshold, stimulus.stair{2}.threshold));    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % run the eye calibration
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
myscreen = eyeCalibDisp(myscreen);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main display loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phaseNum = 1;
while (phaseNum <= length(task)) && ~myscreen.userHitEsc
    % update the task
    [task myscreen phaseNum] = updateTask(task,myscreen,phaseNum);
    % flip screen
    myscreen = tickScreen(myscreen,task);
end


% print out command for next run
disp(sprintf('\n% --------------------------------------------- %\n'));
if stimulus.easyhard == 'hard'
  disp(sprintf('runAlt2Sec(''threshStair1=%0.2f'', ''threshStair2=%0.2f'', ''easyhard=''''easy'''''');', stimulus.stair{1}.threshold, stimulus.stair{2}.threshold));
elseif stimulus.easyhard == 'easy'
  disp(sprintf('runAlt2Sec(''threshStair1=%0.2f'', ''threshStair2=%0.2f'', ''easyhard=''''hard'''''');', stimulus.stair{1}.threshold, stimulus.stair{2}.threshold));
end
disp(sprintf('\n% --------------------------------------------- %\n'));

stimulus.percentCorrect = nansum(stimulus.trialCorrectness)/length(stimulus.trialCorrectness); disp('Percent Correct:'); disp(stimulus.percentCorrect*100);
stimulus.avgRT = nanmean(stimulus.trialRT); disp('Average RT:'); disp(stimulus.avgRT)

% if we got here, we are at the end of the experiment
myscreen = endTask(myscreen,task);
% print out command for next run and run it if numberOfRuns > 1
if iterationCount < numberOfRuns
    iterationCount = iterationCount + 1; 
    threshStair1Str = sprintf('threshStair1=%0.2f',stimulus.stair{1}.threshold);
    threshStair2Str = sprintf('threshStair2=%0.2f',stimulus.stair{2}.threshold);
    iterationCountStr = sprintf('iterationCount=%0.2f',iterationCount);
    disp(sprintf('\n% --------------------------------------------- %\n'));
    if stimulus.easyhard == 'hard'
      disp(sprintf('runAlt2Sec(''threshStair1=%0.2f'', ''threshStair2=%0.2f'', ''easyhard=''''easy'''''');', stimulus.stair{1}.threshold, stimulus.stair{2}.threshold));
      runAlt2Sec(threshStair1Str,threshStair2Str,iterationCountStr,'easyhard=''easy''');
    elseif stimulus.easyhard == 'easy'
      disp(sprintf('runAlt2Sec(''threshStair1=%0.2f'', ''threshStair2=%0.2f'', ''easyhard=''''hard'''''');', stimulus.stair{1}.threshold, stimulus.stair{2}.threshold));
      runAlt2Sec(threshStair1Str,threshStair2Str,iterationCountStr,'easyhard=''hard''');
    end
    disp(sprintf('\n% --------------------------------------------- %\n'));
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets called at the start of each segment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = startSegmentCallback(task, myscreen)
global stimulus;

if task.thistrial.thisseg == stimulus.stimulusSegments
    stimulus.trialCounter = stimulus.trialCounter + 1;
end

% is this a segment in which a stimulus will be displayed
if any(task.thistrial.thisseg == stimulus.stimulusSegments)
    % pick a random stimulus phase
    stimulus.thisPhase = randperm(length(stimulus.phases));
    % which staircase will be updated
    stimulus.whichStair = round(rand)+1;    
    % this is for a hard run
    if strcmp(stimulus.easyhard, 'hard')
        %task.thistrial.thisOrientationStim1 = stimulus.orientation + stimulus.stair{stimulus.whichStair}.threshold * task.thistrial.tiltStim1;
        task.thistrial.thisOrientationStim2 = stimulus.orientation + stimulus.stair{stimulus.whichStair}.threshold * task.thistrial.tiltStim2 * -1;
    elseif strcmp(stimulus.easyhard, 'easy')
        %task.thistrial.thisOrientationStim1 = stimulus.orientation + stimulus.easyTilt * task.thistrial.tiltStim1;
        task.thistrial.thisOrientationStim2 = stimulus.orientation + stimulus.easyTilt * task.thistrial.tiltStim2 * -1;
    end
end

if strcmp(stimulus.easyhard, 'hard')
    stimulus.fixColor = 255.*lab2rgb([53.39 76 62]); % red, 50 lightness in CIELAB -> RGB
else
    stimulus.fixColor = 255.*lab2rgb([53.39 -54 52]); % green, 50 lightness in CIELAB -> RGB
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets called to draw the stimulus each frame
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = screenUpdateCallback(task, myscreen)

global stimulus;
% clear the screen
mglClearScreen;

if any(task.thistrial.thisseg == stimulus.stimulusSegments)
  % mglBltTexture(stimulus.tex(stimulus.thisPhase(1)), [-stimulus.x stimulus.y], 0, 0, task.thistrial.thisOrientationStim1);
  mglBltTexture(stimulus.tex(stimulus.thisPhase(1)), [stimulus.x stimulus.y], 0, 0, task.thistrial.thisOrientationStim2);
  %mglFixationCross(0.7, 2, [0 0 0], [0 0]);
  mglFixationCross(0.7, 2, stimulus.fixColor, [0 0]) % green fixation is cue   
end

if any(task.thistrial.thisseg == 2)
  %mglFixationCross(0.7, 2, [0 0 0], [0 0]);
  mglFixationCross(0.7, 2, stimulus.fixColor, [0 0]) % green fixation is cue   
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets subject response
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = responseCallback(task, myscreen)
global stimulus;

% convert tilt direction(-1,1) into button responses (1,2)
whichTilt = task.thistrial.tiltStim2/2 + 1.5;
% if we are doing the left stimulus (i.e., tiltStim1), we would need to
% reverse, like this whichTilt = -1*task.thistrial.tiltStim1/2 + 1.5;

stimulus.trialResponse(stimulus.trialCounter) = task.thistrial.whichButton;
stimulus.trialCorrectness(stimulus.trialCounter) = task.thistrial.whichButton==whichTilt;
stimulus.trialRT(stimulus.trialCounter) = round(task.thistrial.reactionTime*1000);
stimulus.currTilt(stimulus.trialCounter) = whichTilt;

% make sure we have not already received a response
if task.thistrial.gotResponse==0
    if (task.thistrial.whichButton==whichTilt)
        % play the correct sound
        mglPlaySound('Tink');
        %stimulus.fixColor = [0 0 0];     
        % update staircase 
        if strcmp(stimulus.easyhard, 'hard')
            % update HARD block performance and staircase
            stimulus.stair{stimulus.whichStair} = upDownStaircase(stimulus.stair{stimulus.whichStair}, 1);
        else
            % update EASY block performance
        end
    else
        % play the incorrect sound
        mglPlaySound('Pop');
        %stimulus.fixColor = [0 0 0];     
        % update staircase
        if strcmp(stimulus.easyhard, 'hard')
            % update HARD block performance
            stimulus.stair{stimulus.whichStair} = upDownStaircase(stimulus.stair{stimulus.whichStair}, 0);
        else
            % update EASY block performance
        end
    end
    if strcmp(stimulus.easyhard, 'hard')
        disp(sprintf('%i HARD: Tilt %0.2f, RT: %0.3f, Staircase: %i, Reversals: %i', stimulus.trialCounter, stimulus.stair{stimulus.whichStair}.threshold*task.thistrial.tiltStim2, round(task.thistrial.reactionTime*1000), stimulus.whichStair, stimulus.stair{stimulus.whichStair}.reversaln));
    else
        disp(sprintf('%i EASY: Tilt %0.2f, RT: %0.3f', stimulus.trialCounter, stimulus.easyTilt*task.thistrial.tiltStim2, round(task.thistrial.reactionTime*1000)));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function to init the dot stimulus
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function stimulus = myInitStimulus(stimulus, myscreen, task)

% keep an array that lists which of the segments we
% are presenting the stimulus in.
stimulus.stimulusSegments = 1;

% which phases we will have
stimulus.phases = [0:45:180];

% stimulus size properties (in deg)
stimulus.size = 1;
stimulus.transition = 0.25;
stimulus.textureSize = 1.5;

% make the window for the raised cosine
xDeg2pix = mglGetParam('xDeviceToPixels');
yDeg2pix = mglGetParam('yDeviceToPixels');
widthPixels = round(stimulus.textureSize*xDeg2pix);
heightPixels = round(stimulus.textureSize*yDeg2pix);
widthPixels = widthPixels + mod(widthPixels+1,2);
heightPixels = heightPixels + mod(heightPixels+1,2);
% remember, x and y are flipped 
disc = mkDisc([heightPixels widthPixels], ... % matrix size
    stimulus.size*xDeg2pix/2, ... % radius
    [heightPixels widthPixels]/2, ... % origin
    stimulus.transition * xDeg2pix); % transition


% spatial frequency
stimulus.sf = 4;

% stim location
[stimulus.x, stimulus.y] = pol2cart(-pi/4, 5);
stimulus.orientation = 90;
stimulus.duration = 200;

for i = 1:length(stimulus.phases)
    % make a gabor patch
    grating = mglMakeGrating(stimulus.textureSize, stimulus.textureSize, stimulus.sf, 0, stimulus.phases(i));
    % multiple by the contrast
    grating = grating*stimulus.contrast;
    % multiple by a raised cosine disc
    grating = grating.*disc;
    % scale it to be between 0 and 255
    grating = 255*(grating+1)/2;
    % make it into a texture
    stimulus.tex(i) = mglCreateTexture(grating);
end
